@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use Kwiki::Plugin '-Base';
 use mixin 'Kwiki::Installer';
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 const class_id => 'revisions';
 const class_title => 'Revisions';
@@ -20,30 +20,53 @@ sub register {
     $registry->add(toolbar => 'revisions_controls', 
                    template => 'revisions_controls.html',
                    show_for => 'revisions',
+                   params_class => $self->class_id,
                   );
 }
 
 sub revisions {
+    $self->render_screen($self->toolbar_params);
+}
+
+sub toolbar_params {
     $self->use_class('archive');
     my $page = $self->pages->current;
     $page->load;
     my $revision_id = $self->cgi->revision_id
-      or return $self->redirect($page->id);
+      or return $self->redirect($page->url);
+
+    my $revisions = $page->revision_numbers;
+    $revision_id = $revisions->[-$revision_id] if ($revision_id < 0);
     $self->revision_id($revision_id);
+
     my $archive = $self->hub->load_class('archive');
     $page->content($archive->fetch($page, $revision_id));
-    my ($metadata) = grep {
-        $_->{revision_id} eq $revision_id;
-    } @{$archive->history($page)};
-    $page->metadata->edit_by($metadata->{edit_by});
-    $page->metadata->edit_time($metadata->{edit_time});
-    $page->metadata->edit_unixtime($metadata->{edit_unixtime});
-    $self->page($page);
-    $self->render_screen(
-        screen_title => $page->id,
+    my $page_title = $page->title;
+    my $screen_title = "$page_title <span style=\"font-size:smaller;color:red\">(Revision $revision_id)</span>";
+    my ($prev, $next);
+
+    REVISIONS: {
+        foreach my $index (0..$#$revisions) {
+            $revisions->[$index] == $revision_id or next;
+
+            $prev = $revisions->[$index+1];
+            $next = $revisions->[$index-1] if $index;
+
+            $page->metadata->from_hash(
+                $archive->fetch_metadata($page, $revision_id)
+            );
+
+            last REVISIONS;
+        }
+        die "No such revision: $revision_id";
+    }
+
+    return (
+        screen_title => $screen_title,
         page_html => $page->to_html,
         revision_id => $revision_id,
-        revision_top => $page->revision_number,
+        previous_id => $prev,
+        next_id => $next,
     );
 }
 
@@ -81,33 +104,26 @@ See http://www.perl.com/perl/misc/Artistic.html
 =cut
 __template/tt2/revisions_button.html__
 <!-- BEGIN revisions_button.html -->
-[% revision_number = hub.pages.current.revision_number %]
-[% IF revision_number > 1 %]
-<a href="[% script_name %]?action=revisions&page_id=[% page_id %]&revision_id=[% revision_number - 1 %]" accesskey="r" title="[% revision_number %] Revisions - Link to Previous">
+<a href="[% script_name %]?action=revisions&page_id=[% page_id %]&revision_id=-1" accesskey="r" title="Previous Revision">
 [% INCLUDE revisions_button_icon.html %]
 </a>
-[% END %]
 <!-- END revisions_button.html -->
 __template/tt2/revisions_button_icon.html__
 <!-- BEGIN revisions_button_icon.html -->
-[% revision_number %]&nbsp;Revisions
+Revisions
 <!-- END revisions_button_icon.html -->
 __template/tt2/revisions_controls.html__
 <!-- BEGIN revisions_controls.html -->
-[% id = hub.revisions.cgi.revision_id -%]
-[% top = hub.pages.current.revision_number - 1 -%]
-[% IF id > 1 -%]
-<a href="[% script_name %]?action=revisions&page_id=[% page_id %]&revision_id=[% id - 1 %]" accesskey="p" title="Previous Revision">
+[% IF previous_id -%]
+<a href="[% script_name %]?action=revisions&page_id=[% page_uri %]&revision_id=[% previous_id %]" accesskey="p" title="Previous Revision">
 [% INCLUDE revisions_controls_previous_icon.html %]
-</a>
-|
+</a> | 
 [% END -%]
-<a href="[% script_name %]?[% page_id %]" accesskey="c" title="Current Revision">
+<a href="[% script_name %]?[% page_uri %]" accesskey="c" title="Current Revision">
 [% INCLUDE revisions_controls_current_icon.html %]
 </a>
-[% IF id < top -%]
-|
-<a href="[% script_name %]?action=revisions&page_id=[% page_id %]&revision_id=[% id + 1 %]" accesskey="n" title="Next Revision">
+[% IF next_id -%]
+ | <a href="[% script_name %]?action=revisions&page_id=[% page_uri %]&revision_id=[% next_id %]" accesskey="n" title="Next Revision">
 [% INCLUDE revisions_controls_next_icon.html %]
 </a>
 [% END -%]
@@ -126,7 +142,6 @@ Previous
 <!-- END revisions_controls_previous_icon.html -->
 __template/tt2/revisions_content.html__
 <!-- BEGIN revisions_content.html -->
-[% screen_title = "$page_id <span style=\"font-size:smaller;color:red\">(Revision $revision_id)</span>" -%]
 [% INCLUDE display_changed_by.html %]
 <div class="wiki">
 [% page_html -%]
